package restkiosk.controladors;


import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.pdfbox.exceptions.COSVisitorException;
import org.apache.pdfbox.pdmodel.PDDocument;
import org.apache.pdfbox.pdmodel.PDPage;
import org.apache.pdfbox.pdmodel.edit.PDPageContentStream;
import org.apache.pdfbox.pdmodel.font.PDFont;
import org.apache.pdfbox.pdmodel.font.PDType1Font;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.context.annotation.Configuration;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.ui.ModelMap;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.SessionAttributes;
import org.springframework.web.bind.support.SessionStatus;
import org.springframework.web.servlet.ModelAndView;

import restkiosk.dao.ComandaDao;
import restkiosk.dao.FamiliaDao;
import restkiosk.dao.MenuDao;
import restkiosk.dao.ProducteDao;
import restkiosk.model.Cistella;
import restkiosk.model.Comanda;
import restkiosk.model.ComandaDTO;
import restkiosk.model.Familia;
import restkiosk.model.Menu;
import restkiosk.model.Producte;
import restkiosk.model.ProducteDTO;
import restkiosk.utils.Excepcions;
import flexjson.JSONSerializer;

 

@Controller
@SessionAttributes
@RequestMapping("/public")
@Configuration
@ComponentScan("restkiosk.dao")
public class ControladorComandes  {

	@Autowired
	private FamiliaDao familiaDao;
	@Autowired
	private ProducteDao producteDao;
	@Autowired
	private MenuDao menuDao;
	
	@Autowired
	private ComandaDao comandaDao;
	private Comanda comanda;
	
	@RequestMapping(value ="/mostrarComandes")
	@ExceptionHandler({Excepcions.class})
    public ModelAndView mostrarComandes() {

		try {
			return new ModelAndView("/mostrarComandes");
		} catch (Exception e) {
			// TODO Auto-generated catch block
			throw new Excepcions("Error al mostrar la plana de comandes.");
		}
    }
	
	@ModelAttribute("llistaFamilies")
	@ExceptionHandler({Excepcions.class})
	public List<Familia> getllistaFamilies() {
		try {
			Hashtable<String, Object> filtres = new Hashtable<String, Object>();
			filtres.put(FamiliaDao.FILTER_BY_VISIBLE, true);
		    return familiaDao.llistaFamilies(FamiliaDao.SORT_BY_ORDRE, filtres);
		} catch (Exception e){
			throw new Excepcions("Error al llegir les famlies.");
		}
	}

 	@RequestMapping(value = "/productesFamilia", method = RequestMethod.GET)
 	@ExceptionHandler({Excepcions.class})
    public ModelAndView productesFamilia(@RequestParam("familiaId") Integer familiaId) {
		
 		try {
			Map<String, Object> map = new HashMap<String, Object>();
			List<Producte> productes = producteDao.llistaProductesRelFamilia(familiaId, true, "", 1);
			map.put("productesFamilia", productes);
			return new ModelAndView("/mostrarComandes", map);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			throw new Excepcions("Error al llegir els productes de la famlia sel.leccionada.");
		}
    }
	
 	
 	@RequestMapping(value = "/productesFamiliaJSON",produces="text/plain;charset=UTF-8", method = RequestMethod.GET)
 	@ExceptionHandler({Excepcions.class})
 	public ResponseEntity<String> productesFamiliaJSON(@RequestParam("familiaId") Integer familiaId) {

 	   	try {
			HttpHeaders responseHeaders = new HttpHeaders();
			responseHeaders.add("Content-Type", "text/html; charset=utf-8");
			

			List<Producte> productes = producteDao.llistaProductesRelFamilia(familiaId, true, "", 1);
			JSONSerializer jsonSerializer = new JSONSerializer();
			String json = jsonSerializer.serialize(productes);

			return new ResponseEntity<String>(json, responseHeaders, HttpStatus.OK);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			throw new Excepcions("Error al llegir els productes de la famlia triada.");
		}
 	}
 	
 	@RequestMapping(value = "/menusFamiliaJSON",produces="text/plain;charset=UTF-8", method = RequestMethod.GET)
 	@ExceptionHandler({Excepcions.class})
 	public ResponseEntity<String> menusFamiliaJSON(@RequestParam("familiaId") Integer familiaId) {

 	   	try {
			HttpHeaders responseHeaders = new HttpHeaders();
			responseHeaders.add("Content-Type", "text/html; charset=utf-8");
			
			List<Menu> menus = menuDao.llistaMenusRelFamilia(familiaId, true, "", 1);
			JSONSerializer jsonSerializer = new JSONSerializer();
			String json = jsonSerializer.serialize(menus);

			return new ResponseEntity<String>(json, responseHeaders, HttpStatus.OK);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			throw new Excepcions("Error al llegir els mens de la famlia sel.leccionada.");
		}
 	}
 	
 	@RequestMapping(value = "/productesMenuJSON",produces="text/plain;charset=UTF-8", method = RequestMethod.GET)
 	@ExceptionHandler({Excepcions.class})
 	public ResponseEntity<String> productesMenuJSON(@RequestParam("menuId") Integer menuId) {

 	   	try {
			HttpHeaders responseHeaders = new HttpHeaders();
			responseHeaders.add("Content-Type", "text/html; charset=utf-8");
			

			List<ProducteDTO> productesDTO = new ArrayList<ProducteDTO>();

			List<Producte> productes = producteDao.llistaProductesRelMenu(menuId, true, "", 1, "A");
			Iterator itr = productes.iterator();
			while(itr.hasNext()){
				ProducteDTO pDTO = new ProducteDTO((Producte)itr.next());
				pDTO.setTipus("A");
				productesDTO.add(pDTO);
			}

			productes = producteDao.llistaProductesRelMenu(menuId, true, "", 1, "B");
			itr = productes.iterator();
			while(itr.hasNext()){
				ProducteDTO pDTO = new ProducteDTO((Producte)itr.next());
				pDTO.setTipus("B");
				productesDTO.add(pDTO);
			}

			productes = producteDao.llistaProductesRelMenu(menuId, true, "", 1, "C");
			itr = productes.iterator();
			while(itr.hasNext()){
				ProducteDTO pDTO = new ProducteDTO((Producte)itr.next());
				pDTO.setTipus("C");
				productesDTO.add(pDTO);
			}

			productes = producteDao.llistaProductesRelMenu(menuId, true, "", 1, "D");
			itr = productes.iterator();
			while(itr.hasNext()){
				ProducteDTO pDTO = new ProducteDTO((Producte)itr.next());
				pDTO.setTipus("D");
				productesDTO.add(pDTO);
			}
			
			JSONSerializer jsonSerializer = new JSONSerializer();
			String json = jsonSerializer.serialize(productesDTO);

			return new ResponseEntity<String>(json, responseHeaders, HttpStatus.OK);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			throw new Excepcions("Error al llegir els productes del men sel.leccionat.");
		}
 	}
 	

 	
 	//@Transactional
 	@RequestMapping(value = "/desarCistella", method = RequestMethod.POST)
 	@ExceptionHandler({Excepcions.class})
 	@ResponseBody
    public byte[] afegirProducteCistella(@ModelAttribute("objCistella") Cistella cistella, BindingResult result, SessionStatus status, ModelMap model, 
    		HttpServletRequest request,
    		HttpServletResponse response) {
		
 		try {
			comanda = new Comanda();
			comanda.setDatahora(new Date());
			comandaDao.guardar(comanda);
		} catch (Exception e1) {
			// TODO Auto-generated catch block
			throw new Excepcions("Error a l'afegir producte a la cistella.");
		}
		
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		try {
		  
			PDDocument doc = new PDDocument();
			PDPage page = new PDPage();
			page.setMediaBox(PDPage.PAGE_SIZE_A6);
			doc.addPage(page);
	
			PDFont font = PDType1Font.HELVETICA;
			
			PDPageContentStream contentStream = new PDPageContentStream(doc,page);
			contentStream.beginText();
			contentStream.setFont(font, 10);
			List<ComandaDTO> productes = cistella.getProductesCistella();
			
			float posY = page.getMediaBox().getHeight();//.getUpperRightY();;
			float posX = 0;
			float totalComanda = 0;
			
			contentStream.setTextMatrix(1,0,0,1, 100, 375);
			font = PDType1Font.HELVETICA_BOLD;
			contentStream.setFont(font, 20);
			contentStream.drawString("RestKiosk");

			font = PDType1Font.HELVETICA_BOLD;
			contentStream.setTextMatrix(1,0,0,1, 10, 350);
			contentStream.drawString("Comanda N " + comanda.getIdComanda());

			posY = 300;
			contentStream.setFont(font, 10);
			contentStream.setTextMatrix(1,0,0,1, 20, posY);
			contentStream.drawString("Qt.");
			contentStream.setTextMatrix(1,0,0,1, 40, posY);
			contentStream.drawString("Descripci");
			posX = ((font.getStringWidth("Preu"))/1000)*10;
			contentStream.setTextMatrix(1,0,0,1, 280-posX, posY);
			//contentStream.moveTextPositionByAmount(200, 0);
			contentStream.drawString("Preu");
			posY-=15;

			font = PDType1Font.HELVETICA;
			contentStream.setFont(font, 10);
			String[] descripcio = null;
			for (ComandaDTO prod : productes){
				contentStream.setTextMatrix(1,0,0,1, 20, posY);
				contentStream.drawString(String.valueOf(prod.getQuantitat()));
				
				posX = ((font.getStringWidth(String.valueOf(prod.getPreu())))/1000)*10;
				contentStream.setTextMatrix(1,0,0,1, 280-posX, posY);
				contentStream.drawString(String.valueOf(prod.getPreu()));
				totalComanda += prod.getPreu();

				descripcio = String.valueOf(prod.getDescripcio()).split("::");
				contentStream.setTextMatrix(1,0,0,1, 40, posY);
				contentStream.drawString(descripcio[0]);
				for (int j=1;j<descripcio.length;j++){
					posY-=10;
					contentStream.setTextMatrix(1,0,0,1, 40, posY);
					contentStream.drawString("-"+descripcio[j]);
				}
				
				posY-=12;
				if (posY < 50) {
					contentStream.endText();
					contentStream.close();
					page = new PDPage();
					page.setMediaBox(PDPage.PAGE_SIZE_A6);
					doc.addPage( page );
					contentStream = new PDPageContentStream(doc, page);
					contentStream.beginText();
					contentStream.setFont(font, 10);
					posY = 375;
					font = PDType1Font.HELVETICA_BOLD;
					contentStream.setFont(font, 10);

					contentStream.setTextMatrix(1,0,0,1, 20, posY);
					contentStream.drawString("Qt.");
					contentStream.setTextMatrix(1,0,0,1, 40, posY);
					contentStream.drawString("Descripci");
					posX = ((font.getStringWidth("Preu"))/1000)*10;
					contentStream.setTextMatrix(1,0,0,1, 280-posX, posY);
					//contentStream.moveTextPositionByAmount(200, 0);
					contentStream.drawString("Preu");
					posY-=15;

					font = PDType1Font.HELVETICA;
					contentStream.setFont(font, 10);
				}
			}
			
			posY-=15;
			font = PDType1Font.HELVETICA_BOLD;
			contentStream.setFont(font, 12);
			contentStream.setTextMatrix(1,0,0,1, 10, posY);
			contentStream.drawString("TOTAL: " + String.valueOf(totalComanda));
			posY-=15;
			contentStream.setTextMatrix(1,0,0,1, 10, posY);
			contentStream.drawString("Grcies per la seva visita.");
			contentStream.endText();
			contentStream.close();
			
			//ByteArrayOutputStream baos = new ByteArrayOutputStream();
			doc.save(baos);
			doc.close();

			response.setContentType("application/pdf");
			response.setHeader("Content-Disposition","attachment; filename=\"comanda.pdf\"");
			 
			//OutputStream os = response.getOutputStream();
			//baos.writeTo(os);
			//os.flush();
			//os.close(); 
		} catch (Exception e) {
			throw new Excepcions("Error al generar el ticket reserva.");
		}	
		
		return baos.toByteArray(); 
		//return new ModelAndView("/mostrarComandes");
        //return "redirect:/mostrarFamilies.html";
    }

 
	
	/*@RequestMapping(value = "/imatgeProducteCistella")
	@ResponseBody
	public byte[] mostraImatgeProducte(@RequestParam("producteId") Integer producteId)  {
		producte = producteDao.llegir(producteId);
	  try {
		  return producte.getImatge().getBytes(1, (int)producte.getImatge().length());
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			//e.printStackTrace();
		}
	  	return new byte[0];
	}*/


}
