package restkiosk.controladors;


import java.util.HashMap;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.propertyeditors.CustomCollectionEditor;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.context.annotation.Configuration;
import org.springframework.stereotype.Controller;
import org.springframework.ui.ModelMap;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.WebDataBinder;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.InitBinder;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.SessionAttributes;
import org.springframework.web.bind.support.SessionStatus;
import org.springframework.web.context.request.WebRequest;
import org.springframework.web.servlet.ModelAndView;

import restkiosk.dao.FamiliaDao;
import restkiosk.dao.MenuDao;
import restkiosk.dao.ProducteDao;
import restkiosk.model.Familia;
import restkiosk.model.Menu;
import restkiosk.model.Producte;
import restkiosk.utils.Excepcions;
import restkiosk.validator.FamiliaValidator;

@Controller
@SessionAttributes
@RequestMapping(value = "/admin")
@Configuration
@ComponentScan("restkiosk.dao")
public class ControladorFamilia {

	@Autowired
	private FamiliaDao familiaDao;
	private Familia familia;
	@Autowired
	private ProducteDao producteDao;
	private Producte producte;
	@Autowired
	private MenuDao menuDao;
	private Menu menu;
	
	@RequestMapping("/mostrarFamilies")
	@ExceptionHandler({Excepcions.class})
    public ModelAndView mostrarFamilies(WebRequest webRequest) {
		
		try {
			Hashtable<String, Object> filtres = new Hashtable<String, Object>();
			String descripcio = webRequest.getParameter("descripcio");
			if (descripcio!=null) filtres.put(FamiliaDao.FILTER_BY_DESCRIPCIO, descripcio);	
			String visible = webRequest.getParameter("visible");
			if (visible!=null) 
				if (!visible.equals("")) filtres.put(FamiliaDao.FILTER_BY_VISIBLE, (visible.equals("1")?true:false));
			
			
			
			Map<String, Object> map = new HashMap<String, Object>();
			map.put("descripcio", descripcio);
			map.put("visible", visible);
			map.put("llistaFamilies", familiaDao.llistaFamilies(familiaDao.SORT_BY_DESCRIPCIO, filtres));
			//return "mostrarFamilies";
			return new ModelAndView("/mostrarFamilies",map);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			throw new Excepcions("Error al llegir dades de les famlies.");
		}
    }
	
	@ModelAttribute("llistaMenus")
	@ExceptionHandler({Excepcions.class})
	public List<Menu> getllistaMenus() {
	    try {
			return menuDao.llistaMenusRelFamilia(0, false, "", -1);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			throw new Excepcions("Error al llegir dades de les famlies.");
		}
	}
	
	@ModelAttribute("llistaProductes")
	@ExceptionHandler({Excepcions.class})
	public List<Producte> getllistaProductes() {
	    try {
			return producteDao.llistaProductesRelFamilia(0, false, "", -1);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			throw new Excepcions("Error al llegir dades de les famlies.");
		}
	}
	
	@ModelAttribute("ordrePresentacio")
	@ExceptionHandler({Excepcions.class})
	public List<Familia> getordrePresentacio(){
		try {
			Hashtable<String, Object> filtres = new Hashtable<String, Object>();
			filtres.put(FamiliaDao.FILTER_BY_VISIBLE, true);
			return familiaDao.llistaFamilies(FamiliaDao.SORT_BY_ORDRE, filtres);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			throw new Excepcions("Error al llegir dades de les famlies.");
		}
	}

	@RequestMapping(value = "/editarFamilia", method = RequestMethod.GET)
	@ExceptionHandler({Excepcions.class})
    public ModelAndView editarFamilia(@RequestParam("familiaId") Integer familiaId, Map<String, Object> map) {
		
		 
		
		try {
			if (familiaId==null) familiaId=0;
			
			if (familiaId != 0) {
				familia = familiaDao.llegir(familiaId);
				if (familia==null) familia = new Familia();
			} else {
				familia = new Familia();
			}
			

			map.put("objFamilia", familia);
			//map.put("llistaMenus", producteDao.llistaProductesRelFamilia(0, false, true, ""));
			//map.put("llistaProductes", producteDao.llistaProductesRelFamilia(0, false, false, ""));
			
			//return "editarFamilia";
			return new ModelAndView("/editarFamilia",map);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			throw new Excepcions("Error al editar una famlia.");
		}
    }

    @InitBinder
	@ExceptionHandler({Excepcions.class})
    protected void initBinder(WebDataBinder binder) {
        try {
			binder.registerCustomEditor(Set.class, "productes", new CustomCollectionEditor(Set.class)
			  {
			    @Override
			    protected Object convertElement(Object element)
			    {
			        Integer id = null;

			        if(element instanceof String && !((String)element).equals("")){
			            //From the JSP 'element' will be a String
			            try{
			                id = Integer.parseInt((String) element);
			            }
			            catch (NumberFormatException e) {
			                System.out.println("Element was " + ((String) element));
			                e.printStackTrace();
			            }
			        }
			        else if(element instanceof Integer) {
			            //From the database 'element' will be a Long
			            id = (Integer) element;
			        }

			        return id != null ? producteDao.llegir(id) : null;
			    }
			  });
			
			binder.registerCustomEditor(Set.class, "menus", new CustomCollectionEditor(Set.class)
			{
			  @Override
			  protected Object convertElement(Object element)
			  {
			      Integer id = null;

			      if(element instanceof String && !((String)element).equals("")){
			          //From the JSP 'element' will be a String
			          try{
			              id = Integer.parseInt((String) element);
			          }
			          catch (NumberFormatException e) {
			              System.out.println("Element was " + ((String) element));
			              e.printStackTrace();
			          }
			      }
			      else if(element instanceof Integer) {
			          //From the database 'element' will be a Long
			          id = (Integer) element;
			      }

			      return id != null ? menuDao.llegir(id) : null;
			  }
			});
		} catch (Exception e) {
			// TODO Auto-generated catch block
			throw new Excepcions("Error al recollir dades del formulari de famlies.");
		}
    }		
	
    //@Transactional
    @RequestMapping(value = "/desarFamilia", method = RequestMethod.POST)
	@ExceptionHandler({Excepcions.class})
    public ModelAndView desarFamilia(@ModelAttribute("objFamilia") Familia familia, BindingResult result, SessionStatus status, ModelMap model, WebRequest webRequest) {
		
		try {
			FamiliaValidator familiaValidator = new FamiliaValidator();
			familiaValidator.validate(familia, result);
			
			if(!familia.getDescripcio().equals("")){
				Hashtable<String, Object> filtres = new Hashtable<String, Object>();
				filtres.put(FamiliaDao.FILTER_BY_DESCRIPCIO, familia.getDescripcio());
				filtres.put(FamiliaDao.FILTER_BY_IDDIFERENTDE, familia.getIdFamilia());
				List<Familia> families = familiaDao.llistaFamilies(familiaDao.SORT_BY_DESCRIPCIO, filtres);
				if (families.size()>0) result.rejectValue( "descripcio", "errFamilia.Descripcio.Duplicada");		
			}
			
			Map<String, Object> map = new HashMap<String, Object>();
			if (result.hasErrors()){
				
				String llistaAux = "";
				
				String[] menusDisponibles = webRequest.getParameterValues("menusDisponibles");
				llistaAux = StringUtils.join(menusDisponibles, ",");
			    List<Menu> menus = (List<Menu>)menuDao.llistaMenusRelFamilia(0, false, llistaAux, -1);
			    		
			    if (familia.getMenus()!=null){
			        Iterator itr = menus.iterator();
			        while(itr.hasNext()){
			        	Menu menu = (Menu)itr.next();
			        	boolean trobat = false;
			        	Iterator itr2 = familia.getMenus().iterator();
			        	while (itr2.hasNext() && !trobat){
			        		if (((Menu)itr2.next()).getIdMenu() == menu.getIdMenu()){
				        		menus.remove(menu);
				        		itr = menus.iterator();
			        			trobat = true;
			        		}
			        	}
			        }
			    } 

			    
				String[] productesDisponibles = webRequest.getParameterValues("productesDisponibles");
				llistaAux = StringUtils.join(productesDisponibles, ",");
			    List<Producte> productes = (List<Producte>)producteDao.llistaProductesRelFamilia(0, false, llistaAux, -1);
			    if (familia.getProductes()!=null){
			        Iterator itr = productes.iterator();
			        while(itr.hasNext()){
			        	Producte producte = (Producte)itr.next();
			        	boolean trobat = false;
			        	Iterator itr2 = familia.getProductes().iterator();
			        	while (itr2.hasNext() && !trobat){
			        		if (((Producte)itr2.next()).getIdProducte() == producte.getIdProducte()){
			        			productes.remove(producte);
				        		itr = productes.iterator();
			        			trobat = true;
			        		}
			        	}
			        }
			    }
				map.put("llistaMenus", menus);
				map.put("llistaProductes", productes);
				map.put("missatgeError", true);
				return new ModelAndView("/editarFamilia", map);
			}
			
			familiaDao.guardar(familia);
			/*
			if (familia.getMenus()!=null){
				familia.getMenus().iterator().next().setFamilia_menu(familia);
			    menuDao.guardar(familia.getMenus().iterator().next());
			} 
			if (familia.getProductes()!=null){
				familia.getProductes().iterator().next().setFamilia_producte(familia);
			    producteDao.guardar(familia.getProductes().iterator().next());
			} 
			*/
			String[] ordrePresentacio = webRequest.getParameterValues("ordrePresentacio");
			if (ordrePresentacio!=null) familiaDao.guardarOrdenacio(ordrePresentacio);
			
			status.setComplete();
			//familia = familiaDao.llegir(familia.getIdFamilia());
			//map.put("objFamilia", familia);
			map.put("llistaMenus", menuDao.llistaMenusRelFamilia(0, false, null, -1));
			map.put("llistaProductes", producteDao.llistaProductesRelFamilia(0, false, null, -1));
			map.put("ordrePresentacio", getordrePresentacio());
			map.put("missatgeOk", true);
			return new ModelAndView("/editarFamilia", map);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			throw new Excepcions("Error al desar les dades de la famlia.");
		}
    }
	
    //@Transactional
    @RequestMapping(value = "/esborrarFamilia", method = RequestMethod.POST)
	@ExceptionHandler({Excepcions.class})
    public ModelAndView esborrarFamilia(@ModelAttribute("objFamilia") Familia familia, BindingResult result) {
		
		try {
			Map<String, Object> map = new HashMap<String, Object>();
			
			familiaDao.esborrar(familia.getIdFamilia());
			
			return new ModelAndView("/mostrarFamilies");
		} catch (Exception e) {
			// TODO Auto-generated catch block
			throw new Excepcions("Error a l'esborrar la famlia.");
		}
        
    }
}
