package restkiosk.controladors;


import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.servlet.http.HttpServletRequest;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.propertyeditors.CustomCollectionEditor;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.context.annotation.Configuration;
import org.springframework.stereotype.Controller;
import org.springframework.ui.ModelMap;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.WebDataBinder;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.InitBinder;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.SessionAttributes;
import org.springframework.web.bind.support.SessionStatus;
import org.springframework.web.context.request.WebRequest;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.ModelAndView;

import restkiosk.dao.FamiliaDao;
import restkiosk.dao.MenuDao;
import restkiosk.dao.ProducteDao;
import restkiosk.model.Familia;
import restkiosk.model.Menu;
import restkiosk.model.MenuDTO;
import restkiosk.model.Producte;
import restkiosk.utils.Excepcions;
import restkiosk.validator.MenuValidator;

@Controller
@SessionAttributes
@RequestMapping(value = "/admin")
@Configuration
@ComponentScan("restkiosk.dao")
public class ControladorMenu {

	@Autowired
	private FamiliaDao familiaDao;
	@Autowired
	private MenuDao menuDao;
	private Menu menu;
	@Autowired
	private ProducteDao producteDao;
	
	@RequestMapping("/mostrarMenus")
	@ExceptionHandler({Excepcions.class})
    public ModelAndView mostrarMenus(WebRequest webRequest) {
		
		try {
			Hashtable<String, Object> filtres = new Hashtable<String, Object>();
			String descripcio = webRequest.getParameter("descripcio");
			if (descripcio!=null) filtres.put(FamiliaDao.FILTER_BY_DESCRIPCIO, descripcio);	
			String visible = webRequest.getParameter("visible");
			if (visible!=null) 
				if (!visible.equals("")) filtres.put(FamiliaDao.FILTER_BY_VISIBLE, (visible.equals("1")?"1":"0"));
					
			Map<String, Object> map = new HashMap<String, Object>();
			map.put("descripcio", descripcio);
			map.put("visible", visible);

			map.put("llistaMenus", menuDao.llistaMenus(menuDao.SORT_BY_DESCRIPCIO, filtres));
			//return "mostrarFamilies";
			return new ModelAndView("/mostrarMenus",map);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			throw new Excepcions("Error al llegir dades dels mens.");
		}
    }
	
	@ModelAttribute("llistaFamilies")
	@ExceptionHandler({Excepcions.class})
	public List<Familia> getllistaFamilies() {
	    try {
			return familiaDao.llistaFamilies(FamiliaDao.SORT_BY_DESCRIPCIO, null);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			throw new Excepcions("Error al llegir dades dels mens.");
		}
	}
	
	
	
	@ModelAttribute("ordrePresentacio")
	@ExceptionHandler({Excepcions.class})
	public List<Menu> getordrePresentacio(){
		try {
			Hashtable<String, Object> filtres = new Hashtable<String, Object>();
			filtres.put(MenuDao.FILTER_BY_VISIBLE, "1");
			return menuDao.llistaMenus(MenuDao.SORT_BY_ORDRE, filtres);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			throw new Excepcions("Error al llegir dades dels mens.");

		}
	}

	@RequestMapping(value = "/editarMenu", method = RequestMethod.GET)
	@ExceptionHandler({Excepcions.class})
    public ModelAndView editarMenu(@RequestParam("menuId") Integer menuId, Map<String, Object> map) {
		
		 
		
		try {
			if (menuId==null) menuId=0;
			
			if (menuId != 0) {
				menu = menuDao.llegir(menuId);
				if (menu==null) menu = new Menu();
			} else {
				menu= new Menu();
			}
			
			MenuDTO menuDTO = new MenuDTO(menu);
			menuDTO.setFamiliaMenu(menuDao.llegirFamiliaRelacionada(menu.getIdMenu()));
			menuDTO.setProductesA(producteDao.llistaProductesRelMenu(menuId, true, "", -1, "A"));
			menuDTO.setProductesB(producteDao.llistaProductesRelMenu(menuId, true, "", -1, "B"));
			menuDTO.setProductesC(producteDao.llistaProductesRelMenu(menuId, true, "", -1, "C"));
			menuDTO.setProductesD(producteDao.llistaProductesRelMenu(menuId, true, "", -1, "D"));
			
			List<Producte> llistaProductes = producteDao.llistaProductesRelMenu(menuId, false, "", -1, "");

			map.put("objMenu", menuDTO);
			map.put("llistaProductes", llistaProductes);
			return new ModelAndView("/editarMenu",map);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			throw new Excepcions("Error al editar un men.");
		}
    }

    @InitBinder
    @ExceptionHandler({Excepcions.class})
    protected void initBinder(WebDataBinder binder) {

    	 
        try {
			binder.registerCustomEditor(Set.class, "families", new CustomCollectionEditor(Set.class)
			  {
			    @Override
			    protected Object convertElement(Object element)
			    {
			        Integer id = null;

			        if(element instanceof String && !((String)element).equals("")){
			            //From the JSP 'element' will be a String
			            try{
			                id = Integer.parseInt((String) element);
			            }
			            catch (NumberFormatException e) {
			                System.out.println("Element was " + ((String) element));
			                e.printStackTrace();
			            }
			        }
			        else if(element instanceof Integer) {
			            //From the database 'element' will be a Long
			            id = (Integer) element;
			        }

			        return id != null ? familiaDao.llegir(id) : null;
			    }
			  });
		} catch (Exception e) {
			// TODO Auto-generated catch block
			throw new Excepcions("Error al recollir dades del formulari de mens.");
		}
     
    }
    
    //@Transactional
	@RequestMapping(value = "/desarMenu", method = RequestMethod.POST)
	@ExceptionHandler({Excepcions.class})
    public ModelAndView desarMenu(@ModelAttribute("objMenu") MenuDTO menuDTO , BindingResult result, SessionStatus status, ModelMap model, WebRequest webRequest, HttpServletRequest servletRequest, @RequestParam("fitxer") MultipartFile fitxer) {
		
		try {
			MenuValidator menuValidator = new MenuValidator();
			menuValidator.validate(menuDTO, result);
			
			if (!menuDTO.getTitol().equals("")) {
			Hashtable<String, Object> filtres = new Hashtable<String, Object>();
			filtres.put(FamiliaDao.FILTER_BY_DESCRIPCIO, menuDTO.getTitol());
			filtres.put(FamiliaDao.FILTER_BY_IDDIFERENTDE, menuDTO.getIdMenu());
			List<Menu> menus = menuDao.llistaMenus(MenuDao.SORT_BY_DESCRIPCIO, filtres);
			if (menus.size()>0) result.rejectValue( "titol", "errMenu.titol.Duplicada");
			}
			
			Map<String, Object> map = new HashMap<String, Object>();
			if (result.hasErrors()){
				map.put("missatgeError", true);
				return new ModelAndView("/editarMenu", map);
			}
			
			menu = new Menu(menuDTO);
			
			menuDao.guardar(menu);

			String[] ordrePresentacio = webRequest.getParameterValues("ordrePresentacio");
			if (ordrePresentacio!=null) menuDao.guardarOrdenacio(ordrePresentacio);

			
			//String[] llistaProductes = webRequest.getParameterValues("productesDisponibles");
			String[] productesmenu = webRequest.getParameterValues("llistaproductesA");
			if (productesmenu==null) productesmenu = new String[0]; 
			menuDao.guardarProductes(menuDTO.getIdMenu(), productesmenu, "A");
			productesmenu = webRequest.getParameterValues("llistaproductesB");
			if (productesmenu==null) productesmenu = new String[0]; 
			menuDao.guardarProductes(menuDTO.getIdMenu(), productesmenu, "B");
			productesmenu = webRequest.getParameterValues("llistaproductesC");
			if (productesmenu==null) productesmenu = new String[0];  
			menuDao.guardarProductes(menuDTO.getIdMenu(), productesmenu, "C");
			productesmenu = webRequest.getParameterValues("llistaproductesD");
			if (productesmenu==null) productesmenu = new String[0];  
			menuDao.guardarProductes(menuDTO.getIdMenu(), productesmenu, "D");

			 try {
				 if (fitxer.getSize()>0) {
				 //Blob imatge1 = new SerialBlob(fitxer.getBytes());
				 //menuDTO.setImatge(imatge1);
					 int pos = fitxer.getOriginalFilename().lastIndexOf(".")+1;
					 String nomfitxer = "menu_"+menuDTO.getIdMenu()+"." + fitxer.getOriginalFilename().substring(pos);
					 menuDTO.setImatge(nomfitxer);
					 File f = new File(servletRequest.getRealPath("/")+ File.separator + "imatges" + File.separator + nomfitxer);
					 OutputStream fOut = new FileOutputStream(f);
					 fOut.write(fitxer.getBytes());
					 fOut.close();
					 menuDao.guardarFitxer(menuDTO.getIdMenu(), nomfitxer);
				 
				 } else {
					 //if (menuDTO.getIdMenu()!=0) menuDTO.setImatge(menuDao.llegir(menuDTO.getIdMenu()).getImatge());
				 }
				
			} catch (Exception e) {
				// TODO Auto-generated catch block
				throw new Excepcions("Error al pujar la imatge per al men.");
			//} catch (SerialException e) {
				// TODO Auto-generated catch block
				//e.printStackTrace();
			//} catch (SQLException e) {
				// TODO Auto-generated catch block
				//e.printStackTrace();
			}
			 
			String idfamilia= webRequest.getParameter("families");
			menuDao.desarFamiliaRelacionada(menu.getIdMenu(), Integer.parseInt(idfamilia));
			menuDTO.setFamiliaMenu(menuDao.llegirFamiliaRelacionada(menu.getIdMenu()));
			
			menuDTO.setProductesA(producteDao.llistaProductesRelMenu(menu.getIdMenu(), true, "", -1, "A"));
			menuDTO.setProductesB(producteDao.llistaProductesRelMenu(menu.getIdMenu(), true, "", -1, "B"));
			menuDTO.setProductesC(producteDao.llistaProductesRelMenu(menu.getIdMenu(), true, "", -1, "C"));
			menuDTO.setProductesD(producteDao.llistaProductesRelMenu(menu.getIdMenu(), true, "", -1, "D"));
			
			status.setComplete();
			
			map.put("ordrePresentacio", getordrePresentacio());
			map.put("llistaProductes", producteDao.llistaProductesRelMenu(menu.getIdMenu(), false, "", -1, ""));
			map.put("objMenu", menuDTO);
			
			map.put("missatgeOk", true);
			return new ModelAndView("/editarMenu", map);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			throw new Excepcions("Error al desar les dades del men.");
		}
    }
	
    //@Transactional
	@RequestMapping(value = "/esborrarMenu", method = RequestMethod.POST)
	@ExceptionHandler({Excepcions.class})
    public ModelAndView esborrarMenu(@ModelAttribute("objMenu") Menu menu, BindingResult result) {
		
		try {
			Map<String, Object> map = new HashMap<String, Object>();
			
			menuDao.esborrar(menu.getIdMenu());
			
			return new ModelAndView("/mostrarMenus");
		} catch (Exception e) {
			// TODO Auto-generated catch block
			throw new Excepcions("Error a l'esborrar el men.");
		}
        //return "redirect:/mostrarFamilies.html";
    }
	
	/*@RequestMapping(value = "/imatgeMenu")
	@ResponseBody
	public byte[] mostraImatgeMenu(@RequestParam("menuId") Integer menuId)  {
		menu = menuDao.llegir(menuId);
	  try {
		  return menu.getImatge().getBytes(1, (int)menu.getImatge().length());
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	  	return new byte[0];
	}
	*/
}
