package restkiosk.controladors;


import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.servlet.http.HttpServletRequest;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.propertyeditors.CustomCollectionEditor;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.context.annotation.Configuration;
import org.springframework.stereotype.Controller;
import org.springframework.ui.ModelMap;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.WebDataBinder;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.InitBinder;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.SessionAttributes;
import org.springframework.web.bind.support.SessionStatus;
import org.springframework.web.context.request.WebRequest;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.ModelAndView;

import restkiosk.dao.FamiliaDao;
import restkiosk.dao.ProducteDao;
import restkiosk.model.Familia;
import restkiosk.model.Producte;
import restkiosk.model.ProducteDTO;
import restkiosk.utils.Excepcions;
import restkiosk.validator.ProducteValidator;

@Controller
@SessionAttributes
@RequestMapping(value = "/admin")
@Configuration
@ComponentScan("restkiosk.dao")
public class ControladorProducte {

	@Autowired
	private FamiliaDao familiaDao;
	private Familia familia;
	@Autowired
	private ProducteDao producteDao;
	private Producte producte;
	 
	@RequestMapping("/mostrarProductes")
	@ExceptionHandler({Excepcions.class})
    public ModelAndView mostrarProductes(WebRequest webRequest) {
		
		try {
			Hashtable<String, Object> filtres = new Hashtable<String, Object>();
			String descripcio = webRequest.getParameter("descripcio");
			if (descripcio!=null) filtres.put(FamiliaDao.FILTER_BY_DESCRIPCIO, descripcio);	
			String visible = webRequest.getParameter("visible");
			if (visible!=null) 
				if (!visible.equals("")) filtres.put(FamiliaDao.FILTER_BY_VISIBLE, (visible.equals("1")?"1":"0"));
					
			Map<String, Object> map = new HashMap<String, Object>();
			map.put("descripcio", descripcio);
			map.put("visible", visible);

			map.put("llistaProductes", producteDao.llistaProductes(producteDao.SORT_BY_DESCRIPCIO, filtres));
			//return "mostrarFamilies";
			return new ModelAndView("/mostrarProductes",map);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			throw new Excepcions("Error al llegir dades dels productes.");
		}
    }
	
	@ModelAttribute("llistaFamilies")
	@ExceptionHandler({Excepcions.class})
	public List<Familia> getllistaFamilies() {
	    try {
			return familiaDao.llistaFamilies(familiaDao.SORT_BY_DESCRIPCIO, null);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			throw new Excepcions("Error al llegir dades dels productes.");

		}
	}
	
	@ModelAttribute("ordrePresentacio")
	@ExceptionHandler({Excepcions.class})
	public List<Producte> getordrePresentacio(){
		try {
			Hashtable<String, Object> filtres = new Hashtable<String, Object>();
			filtres.put(producteDao.FILTER_BY_VISIBLE, "1");
			return producteDao.llistaProductes(producteDao.SORT_BY_ORDRE, filtres);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			throw new Excepcions("Error al llegir dades dels productes.");

		}
	}

	@RequestMapping(value = "/editarProducte", method = RequestMethod.GET)
	@ExceptionHandler({Excepcions.class})
    public ModelAndView editarProducte(@RequestParam("producteId") Integer producteId, Map<String, Object> map) {
		
		 
		
		try {
			if (producteId==null) producteId=0;
			
			if (producteId != 0) {
				producte = producteDao.llegir(producteId);
				if (producte==null) producte = new Producte();
			} else {
				producte= new Producte();
			}
			
			ProducteDTO producteDTO = new ProducteDTO(producte);
			producteDTO.setFamiliaProducte(producteDao.llegirFamiliaRelacionada(producte.getIdProducte()));
			
			map.put("objProducte", producteDTO);
			return new ModelAndView("/editarProducte",map);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			throw new Excepcions("Error a l'editar el producte.");
		}
    }

    @InitBinder
    @ExceptionHandler({Excepcions.class})
    protected void initBinder(WebDataBinder binder) {

    	 
        try {
			binder.registerCustomEditor(Set.class, "families", new CustomCollectionEditor(Set.class)
			  {
			    @Override
			    protected Object convertElement(Object element)
			    {
			        Integer id = null;

			        if(element instanceof String && !((String)element).equals("")){
			            //From the JSP 'element' will be a String
			            try{
			                id = Integer.parseInt((String) element);
			            }
			            catch (NumberFormatException e) {
			                System.out.println("Element was " + ((String) element));
			                e.printStackTrace();
			            }
			        }
			        else if(element instanceof Integer) {
			            //From the database 'element' will be a Long
			            id = (Integer) element;
			        }

			        return id != null ? familiaDao.llegir(id) : null;
			    }
			  });
		} catch (Exception e) {
			// TODO Auto-generated catch block
			throw new Excepcions("Error al recollir dades del producte.");
		}
     
    }
    
    //@Transactional
	@RequestMapping(value = "/desarProducte", method = RequestMethod.POST)
	@ExceptionHandler({Excepcions.class})
    public ModelAndView desarProducte(@ModelAttribute("objProducte") ProducteDTO producteDTO , BindingResult result, SessionStatus status, ModelMap model, WebRequest webRequest, HttpServletRequest servletRequest, @RequestParam("fitxer") MultipartFile fitxer) {
		
		try {
			ProducteValidator producteValidator = new ProducteValidator();
			producteValidator.validate(producteDTO, result);
			
			if (!producteDTO.getTitol().equals("")) {
			Hashtable<String, Object> filtres = new Hashtable<String, Object>();
			filtres.put(FamiliaDao.FILTER_BY_DESCRIPCIO, producteDTO.getTitol());
			filtres.put(FamiliaDao.FILTER_BY_IDDIFERENTDE, producteDTO.getIdProducte());
			List<Producte> productes = producteDao.llistaProductes(producteDao.SORT_BY_DESCRIPCIO, filtres);
			if (productes.size()>0) result.rejectValue( "titol", "errProducte.titol.Duplicada");
			}
			
			
			Map<String, Object> map = new HashMap<String, Object>();
			if (result.hasErrors()){
				map.put("missatgeError", true);
				return new ModelAndView("/editarProducte", map);
			}

			producte = new Producte(producteDTO);
			
			producteDao.guardar(producte);

			String[] ordrePresentacio = webRequest.getParameterValues("ordrePresentacio");
			if (ordrePresentacio!=null) producteDao.guardarOrdenacio(ordrePresentacio);
			
			 try {
				 if (fitxer.getSize()>0) {
				 //Blob imatge1 = new SerialBlob(fitxer.getBytes());
				 //producteDTO.setImatge(imatge1);
				 
				 int pos = fitxer.getOriginalFilename().lastIndexOf(".")+1;
				 String nomfitxer = "prod_"+producteDTO.getIdProducte()+"." + fitxer.getOriginalFilename().substring(pos);
				 producteDTO.setImatge(nomfitxer);
				 File f = new File(servletRequest.getRealPath("/")+ File.separator + "imatges" + File.separator + nomfitxer);
				 OutputStream fOut = new FileOutputStream(f);
				 fOut.write(fitxer.getBytes());
				 fOut.close();
				 producteDao.guardarFitxer(producteDTO.getIdProducte(), nomfitxer);
				 
				 } else {
					 //if (producteDTO.getIdProducte()!=0) producteDTO.setImatge(producteDao.llegir(producteDTO.getIdProducte()).getImatge());
				 }
				
			} catch (Exception e) {
				// TODO Auto-generated catch block
				throw new Excepcions("Error al pujar la imatge per al men.");
			//} catch (SerialException e) {
				// TODO Auto-generated catch block
				//e.printStackTrace();
			//} catch (SQLException e) {
				// TODO Auto-generated catch block
				//e.printStackTrace();
			}
			 
			String idfamilia= webRequest.getParameter("families");
			producteDao.desarFamiliaRelacionada(producte.getIdProducte(), Integer.parseInt(idfamilia));
			producteDTO.setFamiliaProducte(producteDao.llegirFamiliaRelacionada(producte.getIdProducte()));
			status.setComplete();
			
			map.put("ordrePresentacio", getordrePresentacio());
			map.put("missatgeOk", true);
			return new ModelAndView("/editarProducte", map);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			throw new Excepcions("Error al desar el producte.");
		}
    }
	
    //@Transactional
	@RequestMapping(value = "/esborrarProducte", method = RequestMethod.POST)
	@ExceptionHandler({Excepcions.class})
    public ModelAndView esborrarProducte(@ModelAttribute("objProducte") Producte producte, BindingResult result) {
		
		try {
			Map<String, Object> map = new HashMap<String, Object>();
			
			producteDao.esborrar(producte.getIdProducte());
			
			return new ModelAndView("/mostrarProductes");
		} catch (Exception e) {
			// TODO Auto-generated catch block
			throw new Excepcions("Error a l'esborrar el producte.");
		}
        //return "redirect:/mostrarFamilies.html";
    }
	
	/*@RequestMapping(value = "/imatgeProducte")
	@ResponseBody
	public byte[] mostraImatgeProducte(@RequestParam("producteId") Integer producteId)  {
		producte = producteDao.llegir(producteId);
	  try {
		  if (producte.getImatge()!=null) return producte.getImatge().getBytes(1, (int)producte.getImatge().length());
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	  	return new byte[0];
	}*/
}
