package restkiosk.dao;

import java.util.Hashtable;
import java.util.List;

import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.ExceptionHandler;

import restkiosk.model.Familia;
import restkiosk.model.Menu;
import restkiosk.model.Producte;
import restkiosk.utils.Excepcions;


@Repository
public class FamiliaDao {

	@Autowired
    private SessionFactory sessionFactory;

	
    @Autowired
    public void setSessionFactory(SessionFactory sessionFactory) {
        this.sessionFactory = sessionFactory;
    }
    
	public static final String SORT_BY_ORDRE = "ORDRE";
	public static final String SORT_BY_DESCRIPCIO = "DESCRIPCIO";

	public static final String FILTER_BY_DESCRIPCIO = "DESCRIPCIO";
	public static final String FILTER_BY_VISIBLE = "VISIBLE";
	public static final String FILTER_BY_IDDIFERENTDE = "IDDIFERENTDE";

    @Transactional
    @ExceptionHandler({Excepcions.class})
    public List<Familia> llistaFamilies(String ordre, Hashtable<String, Object> filtres) {
        Session session = sessionFactory.getCurrentSession();
    	
        List<Familia> families = null;
        try {
        	String sql = "select {fam.*} from families fam ";
        	
        	String sqlWhere = "";
        	if (filtres != null){
       			if (filtres.containsKey(FILTER_BY_DESCRIPCIO) && filtres.containsKey(FILTER_BY_IDDIFERENTDE)) sqlWhere += (sqlWhere.equals("")?"":" AND ") + " fam.Descripcio = :descripcio ";
       			else if (filtres.containsKey(FILTER_BY_DESCRIPCIO)) sqlWhere += (sqlWhere.equals("")?"":" AND ") + " fam.Descripcio LIKE :descripcio ";
       			if (filtres.containsKey(FILTER_BY_VISIBLE)) sqlWhere += (sqlWhere.equals("")?"":" AND ") + " fam.Visible= :visible ";
       			if (filtres.containsKey(FILTER_BY_IDDIFERENTDE)) sqlWhere += (sqlWhere.equals("")?"":" AND ") + " fam.IdFamilia<>:idfam ";
        	}
        	
        	if (!sqlWhere.equals("")) sql += " WHERE " + sqlWhere;
        		
        	if (ordre == null) ordre=SORT_BY_DESCRIPCIO;
        	if (ordre.equals(SORT_BY_DESCRIPCIO)) sql += " order by fam.Descripcio asc ";
        	else if (ordre.equals(SORT_BY_ORDRE)) sql += " order by fam.Ordre asc, fam.Descripcio asc ";

        	Query query = session.createSQLQuery(sql).addEntity("fam", Familia.class);
        	
        	if (filtres != null){
        		if (filtres.containsKey(FILTER_BY_DESCRIPCIO) && filtres.containsKey(FILTER_BY_IDDIFERENTDE)) query.setString("descripcio", ""+filtres.get(FILTER_BY_DESCRIPCIO)+"");
        		else if (filtres.containsKey(FILTER_BY_DESCRIPCIO)) query.setString("descripcio", "%"+filtres.get(FILTER_BY_DESCRIPCIO)+"%");
        		if (filtres.containsKey(FILTER_BY_VISIBLE)) query.setBoolean("visible", Boolean.parseBoolean(String.valueOf(filtres.get(FILTER_BY_VISIBLE))));
        		if (filtres.containsKey(FILTER_BY_IDDIFERENTDE)) query.setLong("idfam", Long.parseLong(String.valueOf(filtres.get(FILTER_BY_IDDIFERENTDE))));
        	}
        	
        	families = (List<Familia>)query.list();
        	
        } catch (HibernateException e) {
        	throw new Excepcions("Error al llegir les famlies.");
        } finally {
        	
        }
        return families;
    }
    
    
    @Transactional
    public Familia llegir(Integer id) {
    	Session session = sessionFactory.getCurrentSession();
    	//this.session = HibernateUtils.getSessionFactory().openSession();
    	Familia obj = (Familia)session.get(Familia.class, id);
    	//this.session.close();
        return obj;
        
    }
    
    @Transactional
    public void guardar(Familia familia) {
    	Session session = sessionFactory.getCurrentSession();
    	//this.session = HibernateUtils.getSessionFactory().openSession();
        
        session.saveOrUpdate(familia);
    	//this.session.close();

    }
    
    @Transactional
    public void esborrar(Integer id) {
    	Session session = sessionFactory.getCurrentSession();
    	//this.session = HibernateUtils.getSessionFactory().openSession();
         
        Familia familia = (Familia)session.get(Familia.class, id);
        Menu menu = null;
        while( familia.getMenus().iterator().hasNext()){
        	menu = (Menu) familia.getMenus().iterator().next();
        	familia.getMenus().remove(menu);
        }
        Producte prod = null;
        while( familia.getProductes().iterator().hasNext()){
        	prod = (Producte) familia.getProductes().iterator().next();
        	familia.getProductes().remove(prod);
        }
        
        session.delete(familia);
    	//this.session.close();

    }
    
    @Transactional
    public void guardarOrdenacio(String[] ids){
   	 	Session session = sessionFactory.getCurrentSession();
   	//this.session = HibernateUtils.getSessionFactory().openSession();
    	
    	for(int i=0; i < ids.length; i++){
    		session.createSQLQuery("UPDATE families SET Ordre="+i+" WHERE IdFamilia="+ids[i]).executeUpdate();
    	}
    	//this.session.close();

    	
    }

}
