package restkiosk.dao;

import java.util.Hashtable;
import java.util.List;

import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.ExceptionHandler;


import restkiosk.model.Familia;
import restkiosk.model.Menu;
import restkiosk.utils.Excepcions;


@Repository
public class MenuDao {

	@Autowired
    private SessionFactory sessionFactory;
	
    @Autowired
    public void setSessionFactory(SessionFactory sessionFactory) {
        this.sessionFactory = sessionFactory;
    }
    
    
	public static final String SORT_BY_ORDRE = "ORDRE";
	public static final String SORT_BY_DESCRIPCIO = "DESCRIPCIO";

	public static final String FILTER_BY_DESCRIPCIO = "DESCRIPCIO";
	public static final String FILTER_BY_VISIBLE = "VISIBLE";
	public static final String FILTER_BY_IDDIFERENTDE = "IDDIFERENTDE";
	
    @Transactional
    @ExceptionHandler({Excepcions.class})
    public List<Menu> llistaMenus(String ordre, Hashtable<String, Object> filtres) {
    	Session session = sessionFactory.getCurrentSession();
        List<Menu> menus = null;
        try {
        	String sql = "select {men.*} from menus men ";
        	
        	String sqlWhere = "";
        	if (filtres != null){
        		if (filtres.containsKey(FILTER_BY_DESCRIPCIO) && filtres.containsKey(FILTER_BY_IDDIFERENTDE)) sqlWhere += (sqlWhere.equals("")?"":" AND ") + " men.Titol = :titol ";
        		else if (filtres.containsKey(FILTER_BY_DESCRIPCIO)) sqlWhere += (sqlWhere.equals("")?"":" AND ") + " men.Titol LIKE :titol ";
       			if (filtres.containsKey(FILTER_BY_VISIBLE)) sqlWhere += (sqlWhere.equals("")?"":" AND ") + " men.Visible= :visible ";
       			if (filtres.containsKey(FILTER_BY_IDDIFERENTDE)) sqlWhere += (sqlWhere.equals("")?"":" AND ") + " men.IdMenu<>:idmen ";
        	}
        	
        	if (!sqlWhere.equals("")) sql += " WHERE " + sqlWhere;
        		
        	if (ordre == null) ordre=SORT_BY_DESCRIPCIO;
        	if (ordre.equals(SORT_BY_DESCRIPCIO)) sql += " order by men.Titol asc ";
        	else if (ordre.equals(SORT_BY_ORDRE)) sql += " order by men.Ordre asc, men.Titol asc ";

        	Query query = session.createSQLQuery(sql).addEntity("men", Menu.class);
        	
        	if (filtres != null){
        		if (filtres.containsKey(FILTER_BY_DESCRIPCIO) && filtres.containsKey(FILTER_BY_IDDIFERENTDE)) query.setString("titol", "%"+filtres.get(FILTER_BY_DESCRIPCIO)+"%");
        		else if (filtres.containsKey(FILTER_BY_DESCRIPCIO)) query.setString("titol", "%"+filtres.get(FILTER_BY_DESCRIPCIO)+"%");
        		if (filtres.containsKey(FILTER_BY_VISIBLE)) query.setInteger("visible", Integer.parseInt(String.valueOf(filtres.get(FILTER_BY_VISIBLE))));
        		if (filtres.containsKey(FILTER_BY_IDDIFERENTDE)) query.setLong("idmen", Long.parseLong(String.valueOf(filtres.get(FILTER_BY_IDDIFERENTDE))));        		
        	}
        	
        	menus = (List<Menu>)query.list();
        	
        } catch (HibernateException e) {
        	throw new Excepcions("Error al llegir els mens.");
        }
        return menus;
    }
    
    @Transactional
    @ExceptionHandler({Excepcions.class})
    public List<Menu> llistaMenusRelFamilia(Integer idFamilia, boolean assignats, String filtreIds, int visible) {
        Session session = sessionFactory.getCurrentSession();
        List<Menu> menus = null;
        try {
        	String sql = "";
        	
            	if (!assignats) { 
            		sql = "select {men.*} from menus men where  ";
            		if (filtreIds == null) filtreIds = "";
            		if (filtreIds.equals("")) {
            			sql += " men.IdMenu not in (select frm.IdMenu from familiesrelmenus frm ) ";	
            			//sql += " men.IdFamilia IS NULL  ";
            		} else {
            			sql += " men.IdMenu in (" + filtreIds + ") ";
            		}
            		if (visible!=-1) sql+=" AND prod.Visible="+visible;
            		
            		sql += " order by men.Titol asc";
            	}
            	if (assignats) 
            		sql = "select {men.*} from menus men, familiesrelmenus frp where men.IdMenu=frp.IdMenu " + (visible!=-1?" AND men.Visible="+visible:"")+ " and frp.IdFamilia="+ idFamilia + " order by men.Ordre asc, men.Titol";
            		//sql = "select {men.*} from Menus men where men.IdFamilia="+ idFamilia + " order by men.titol asc";
            		
        		
        	menus = (List<Menu>)session.createSQLQuery(sql).addEntity("men", Menu.class).list();
        	
        } catch (HibernateException e) {
        	throw new Excepcions("Error al llegir la relaci entre mens/famlies.");
        }
        return menus;
    }

    
    @Transactional
    public Menu llegir(Integer id) {
        Session session = sessionFactory.getCurrentSession();
        return (Menu)session.get(Menu.class, id);
    }
    
    @Transactional
    public void guardar(Menu menu) {
        Session session = sessionFactory.getCurrentSession();
        session.saveOrUpdate(menu);
    }
    
    @Transactional
    public void esborrar(Integer id) {
        Session session = sessionFactory.getCurrentSession();
         
        session.createSQLQuery("DELETE FROM familiesrelmenus WHERE IdMenu=" + id).executeUpdate();
        session.createSQLQuery("DELETE FROM menusrelproductes WHERE IdMenu=" + id).executeUpdate();
        Menu menu = (Menu)session.get(Menu.class, id);
         
        session.delete(menu);
    }
    
    @Transactional
    public void guardarFitxer(Integer id, String fitxer){
   	 	Session session = sessionFactory.getCurrentSession();
   	 	session.createSQLQuery("UPDATE menus SET Imatge='"+fitxer+"' WHERE IdMenu="+id).executeUpdate();
    	
    }
    
    @Transactional
    public void guardarOrdenacio(String[] ids){
   	 	Session session = sessionFactory.getCurrentSession();
    	for(int i=0; i < ids.length; i++){
    		session.createSQLQuery("UPDATE menus SET Ordre="+i+" WHERE IdMenu="+ids[i]).executeUpdate();
    	}
    }

    @Transactional
    public void guardarProductes(Integer id, String[] ids, String tipus){
   	 	Session session = sessionFactory.getCurrentSession();
		session.createSQLQuery("DELETE FROM menusrelproductes WHERE IdMenu="+id+" AND Tipus='"+tipus+"'").executeUpdate();
    	for(int i=0; i < ids.length; i++){
    		session.createSQLQuery("INSERT INTO menusrelproductes (IdMenu, IdProducte, Tipus) VALUES (" + id + ","+ ids[i] + ",'" + tipus + "')").executeUpdate();
    	}
    }

    @Transactional
    public Familia llegirFamiliaRelacionada(Integer idmenu){
   	 	Session session = sessionFactory.getCurrentSession();
		List<Familia> familiamenu = (List<Familia>)session.createSQLQuery("SELECT {fam.*} FROM families fam WHERE fam.IdFamilia IN (SELECT frp.IdFamilia FROM familiesrelmenus frp WHERE frp.IdMenu=" + idmenu +")").addEntity("fam", Familia.class).list();
		if (familiamenu!=null) {
			if (familiamenu.size()>0) return familiamenu.get(0);
			else return new Familia();
		} else {
			return new Familia();
		}
		
    }
    @Transactional
    public void desarFamiliaRelacionada(Integer idmenu, Integer idfamilia){
   	 	Session session = sessionFactory.getCurrentSession();
   	 	//if (idfamilia == 0) session.createSQLQuery("DELETE FROM FamiliesRelMenus WHERE IdMenu=" + idmenu + " AND IdFamilia="+idfamilia).executeUpdate();
   	 	//else session.createSQLQuery("UPDATE FamiliesRelMenus SET IdFamilia=" + idfamilia + " WHERE IdMenu=" + idmenu).executeUpdate();
   	 	session.createSQLQuery("DELETE FROM familiesrelmenus WHERE IdMenu=" + idmenu).executeUpdate();
   	 	session.createSQLQuery("INSERT INTO familiesrelmenus (IdFamilia, IdMenu) VALUES(" + idfamilia + "," + idmenu+")").executeUpdate();   	 	
    }
    
}
