package restkiosk.dao;

import java.util.Hashtable;
import java.util.List;

import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.ExceptionHandler;


import restkiosk.model.Familia;
import restkiosk.model.Producte;
import restkiosk.utils.Excepcions;

@Repository
public class ProducteDao {

	@Autowired
    private SessionFactory sessionFactory;
	
    @Autowired
    public void setSessionFactory(SessionFactory sessionFactory) {
        this.sessionFactory = sessionFactory;
    }
    
    
	public static final String SORT_BY_ORDRE = "ORDRE";
	public static final String SORT_BY_DESCRIPCIO = "DESCRIPCIO";

	public static final String FILTER_BY_DESCRIPCIO = "DESCRIPCIO";
	public static final String FILTER_BY_VISIBLE = "VISIBLE";
	public static final String FILTER_BY_IDDIFERENTDE = "IDDIFERENTDE";
	
    @Transactional
    @ExceptionHandler({Excepcions.class})
    public List<Producte> llistaProductes(String ordre, Hashtable<String, Object> filtres) {
    	Session session = sessionFactory.getCurrentSession();
        List<Producte> productes = null;
        try {
        	String sql = "select prod.* from productes prod ";
        	
        	String sqlWhere = "";
        	if (filtres != null){
       			if (filtres.containsKey(FILTER_BY_DESCRIPCIO) && filtres.containsKey(FILTER_BY_IDDIFERENTDE)) sqlWhere += (sqlWhere.equals("")?"":" AND ") + " prod.Titol = :titol ";
       			else if (filtres.containsKey(FILTER_BY_DESCRIPCIO) ) sqlWhere += (sqlWhere.equals("")?"":" AND ") + " prod.Titol LIKE :titol "; 
       			if (filtres.containsKey(FILTER_BY_VISIBLE)) sqlWhere += (sqlWhere.equals("")?"":" AND ") + " prod.Visible= :visible ";
      			if (filtres.containsKey(FILTER_BY_IDDIFERENTDE)) sqlWhere += (sqlWhere.equals("")?"":" AND ") + " prod.IdProducte<>:idprod ";
        	}
        	
        	if (!sqlWhere.equals("")) sql += " WHERE " + sqlWhere;
        		
        	if (ordre == null) ordre=SORT_BY_DESCRIPCIO;
        	if (ordre.equals(SORT_BY_DESCRIPCIO)) sql += " order by prod.Titol asc ";
        	else if (ordre.equals(SORT_BY_ORDRE)) sql += " order by prod.Ordre asc, prod.Titol asc ";

        	Query query = session.createSQLQuery(sql).addEntity("prod", Producte.class);
        	
        	if (filtres != null){
        		if (filtres.containsKey(FILTER_BY_DESCRIPCIO) && filtres.containsKey(FILTER_BY_IDDIFERENTDE)) query.setString("titol", ""+filtres.get(FILTER_BY_DESCRIPCIO)+"");
        		else if (filtres.containsKey(FILTER_BY_DESCRIPCIO)) query.setString("titol", "%"+filtres.get(FILTER_BY_DESCRIPCIO)+"%");
        		if (filtres.containsKey(FILTER_BY_VISIBLE)) query.setInteger("visible", Integer.parseInt(String.valueOf(filtres.get(FILTER_BY_VISIBLE))));
        		if (filtres.containsKey(FILTER_BY_IDDIFERENTDE)) query.setLong("idprod", Long.parseLong(String.valueOf(filtres.get(FILTER_BY_IDDIFERENTDE))));        		

        	}
        	
        	productes = (List<Producte>)query.list();
        	
        } catch (HibernateException e) {
        	throw new Excepcions("Error al llegir la llista de productes.");
        }
        return productes;
    }
    
    @Transactional
    @ExceptionHandler({Excepcions.class})
    public List<Producte> llistaProductesRelFamilia(Integer idFamilia, boolean assignats, String filtreIds, int visible) {
        Session session = sessionFactory.getCurrentSession();
        List<Producte> productes = null;
        try {
        	String sql = "";
        		
            	if (!assignats) {
            		sql = "select prod.* from productes prod where ";

            		if (filtreIds == null) filtreIds = "";
            		if (filtreIds.equals("")) {
            			sql += " prod.IdProducte not in (select frp.IdProducte from familiesrelproductes frp) ";	
            			//sql += " prod.IdFamilia IS NULL ";
            		} else {
            			sql += " prod.IdProducte in (" + filtreIds + ") ";
            		}
            		if (visible!=-1) sql+=" AND prod.Visible="+visible;
            			
            		
            		sql += "order by prod.Titol asc";
            	}
            	if (assignats) 
            		sql = "select {prod.*} from productes prod, familiesrelproductes frp where prod.IdProducte=frp.IdProducte " + (visible!=-1?" AND prod.Visible="+visible:"")+ " and frp.IdFamilia="+ idFamilia + " order by prod.Ordre asc, prod.Titol";
            		//sql = "select {prod.*} from Productes prod where Prod.IdFamilia="+ idFamilia + " order by prod.titol asc";
            	
            	
            	productes = (List<Producte>)session.createSQLQuery(sql).addEntity("prod", Producte.class).list();
        	
        } catch (HibernateException e) {
        	throw new Excepcions("Error al llegir la relaci entre productes/famlies.");
        }
        return productes;
    }

    @Transactional
    @ExceptionHandler({Excepcions.class})
    public List<Producte> llistaProductesRelMenu(Integer idMenu, boolean assignats, String filtreIds, int visible, String tipus) {
        Session session = sessionFactory.getCurrentSession();
        List<Producte> productes = null;
        try {
        	String sql = "";
        		
            	if (!assignats) {
            		sql = "select prod.* from productes prod where ";

            		if (filtreIds == null) filtreIds = "";
            		if (filtreIds.equals("")) {
            			sql += " prod.IdProducte not in (select mrp.IdProducte from menusrelproductes mrp where mrp.IdMenu="+idMenu+") ";	
            			//sql += " prod.IdFamilia IS NULL ";
            		} else {
            			sql += " prod.IdProducte in (" + filtreIds + ") ";
            		}
            		if (visible!=-1) sql+=" AND prod.Visible="+visible;
            			
            		
            		sql += "order by prod.Titol asc";
            	}
            	if (assignats) 
            		sql = "select {prod.*} from productes prod, menusrelproductes mrp where prod.IdProducte=mrp.IdProducte and mrp.IdMenu="+idMenu+ " " + (tipus!=null?" AND mrp.Tipus='"+tipus+"'":"") + (visible!=-1?" AND prod.Visible="+visible:"")+ " order by mrp.Tipus asc, prod.Ordre asc, prod.Titol";
            		//sql = "select {prod.*} from Productes prod where Prod.IdFamilia="+ idFamilia + " order by prod.titol asc";
            	
            	
            	productes = (List<Producte>)session.createSQLQuery(sql).addEntity("prod", Producte.class).list();
        	
        } catch (HibernateException e) {
        	throw new Excepcions("Error al llegir la relaci entre productes i mens.");
        }
        return productes;
    }

    
    @Transactional
    public Producte llegir(Integer id) {
        Session session = sessionFactory.getCurrentSession();
        return (Producte)session.get(Producte.class, id);
    }
    
    @Transactional
    public void guardar(Producte producte) {
        Session session = sessionFactory.getCurrentSession();
        session.saveOrUpdate(producte);
    }
    
    @Transactional
    public void guardarFitxer(Integer id, String fitxer){
   	 	Session session = sessionFactory.getCurrentSession();
   	 	session.createSQLQuery("UPDATE productes SET Imatge='"+fitxer+"' WHERE IdProducte="+id).executeUpdate();
    	
    }
    
    @Transactional
    public void esborrar(Integer id) {
        Session session = sessionFactory.getCurrentSession();
        
        session.createSQLQuery("DELETE FROM familiesrelproductes WHERE IdProducte=" + id).executeUpdate();
        Producte producte = (Producte)session.get(Producte.class, id);
         
        session.delete(producte);
    }
    
    @Transactional
    public void guardarOrdenacio(String[] ids){
   	 	Session session = sessionFactory.getCurrentSession();
    	for(int i=0; i < ids.length; i++){
    		session.createSQLQuery("UPDATE productes SET Ordre="+i+" WHERE IdProducte="+ids[i]).executeUpdate();
    	}
    }
    
    @Transactional
    public Familia llegirFamiliaRelacionada(Integer idproducte){
   	 	Session session = sessionFactory.getCurrentSession();
		List<Familia> familiaproducte = (List<Familia>)session.createSQLQuery("SELECT {fam.*} FROM families fam WHERE fam.IdFamilia IN (SELECT frp.IdFamilia FROM familiesrelproductes frp WHERE frp.IdProducte=" + idproducte +")").addEntity("fam", Familia.class).list();
		if (familiaproducte!=null) {
			if (familiaproducte.size()>0) return familiaproducte.get(0);
			else return new Familia();
		} else {
			return new Familia();
		}
		
    }
    @Transactional
    public void desarFamiliaRelacionada(Integer idproducte, Integer idfamilia){
   	 	Session session = sessionFactory.getCurrentSession();
   	 	session.createSQLQuery("DELETE FROM familiesrelproductes WHERE IdProducte=" + idproducte).executeUpdate();
   	 	session.createSQLQuery("INSERT INTO familiesrelproductes (IdFamilia, IdProducte) VALUES(" + idfamilia + "," + idproducte+")").executeUpdate();
    }
}
